
######################################################################
#  FIT A FAR(2) model:  this fit needs two functions
#                far and mcv
#  mcv is the function used to select bandwidth by using multifold cv
#  far is used to compute fitted coefficients with a given h   
#  The Epanechinikov kernel is used                                  #
#  Result:  fitted coefficients and residuals 
######################################################################

######################################################## 
# Define the local linear and constant fitting         #
########################################################
far<-function(y,x1,x2,u,u0,h,method=1)
  { #for given points u0, use local linear or constant to find FAR(2) fit
    #method = 1, local linear fit; method = 0, local constant fit
    #data: y, x1, x2, u
    ngrid<-length(u0)
    beta<-matrix(0,ncol=2,nrow=ngrid)
    for (k in 1:ngrid)
     {xu1<-x1*(u-u0[k])
      xu2<-x2*(u-u0[k])
      w0 <- (1 - (u-u0[k])^2/h^2)*(abs(u-u0[k]) <= h) + 0.0001                  
      mydata<-data.frame(y,x1,x2,xu1,xu2,w0)
      if(method == 1)
       {fit<-glm(y~-1+x1+x2+xu1+xu2,family=gaussian,data=mydata,weights=w0)
        beta[k,]<-coef(fit)[1:2]
       }
      else
       {fit<-glm(y~-1+x1+x2,family=gaussian,data=mydata,weights=w0)
        beta[k,]<-coef(fit)[1:2]
       }
      }
     beta
     a1 <- approx(u0, beta[,1], u, rule=2)$y
     a2 <- approx(u0, beta[,2], u, rule=2)$y
     yhat <- a1 * x1 + a2 * x2                   # fitted values
     list(coef=beta, resid =  y - yhat)
  }



  

##################################################################   
#   mcv:  multifold cross-validation used for selecting bandwidth
##################################################################
mcv <- function(y,x1,x2,u,method=1)
 { #multi-fold cross-validation for computing optimal bandwidth.
   #parameters are defined below.
   n<-length(y)
   m<-floor(0.1*n)
   h0<- seq(0.3, 3,0.3) *sqrt(var(u0))
   nh<-length(h0)
   Q<-4
   ams<-matrix(0,nh,Q)

  for(qq in 1:Q)
  { print(c("q=", qq))
    n1<-n-qq*m
    yq<-y[1:n1]
    x1q<-x1[1:n1]
    uq<-u[1:n1]
    x2q<-x2[1:n1]
    u0q<-u[(n1+1):(n1+m)]
    for(hh in 1:nh)
     {
      h<-h0[hh]*(n/n1)^0.2                 # normalized bandwidth
      beta<-far(yq,x1q,x2q,uq,u0q,h,method)  
                                  # compute the estimate of a_1(u) and a_2(u)
      yhat<-beta[,1]*x1[(n1+1):(n1+m)]+beta[,2]*x2[(n1+1):(n1+m)] 
      				  # fitted vaules
      ams[hh,qq]<-mean((y[(n1+1):(n1+m)]-yhat)^2)              # AMS_q
     }    # end of loop for bandwidth search
  }       # end of loop for Q
  amsh<-apply(ams,1,sum)
  index<-order(amsh)
  hopt<-h0[index[1]]
  print(c("THE OPTIMAL BANDWIDTH IS", hopt))
  #write(t(cbind(h0,amsh)),"fanh.out",ncol=2)
 } 
